
capture program drop callagain
*! callagain v1.0 CBasavaraj Sep2014
*! callagain v1.1 LBehaghel Sep2015
*! callagain v1.2 TLeBarbanchon Dec2015
program callagain, eclass
	version 10
	
	
* **** 1. Syntax
	
	local cmd "callagain"
	local cmdline "`cmd' `*'"
	
	syntax varlist(min=4 max=4 numeric) [if] [in] [fw aw], ytype(string) [Graph] [LEVel(real 95)]
	
	marksample touse, novarlist

	
	//install invcdf 
	*(for calculating y(alpha) when ytype == "cont")
	capture which invcdf
	if _rc==111 ssc install invcdf
	
	//more
	local cmore `c(more)'
	set more off
	
	//level
	local clevel = `c(level)'
	if `level' >= 50 & `level' < 100 {
		local level = round(`level',0.01)
		set level `level'
	}
	else {
		local level = `clevel'			
		// set level `clevel'
	}
	
	//varlist
	tokenize `varlist'
	local y `1'
	local Z `2'
	local R `3'
	local N `4'
	
	if "`ytype'" != "bin" & "`ytype'" != "cont" {
		dis as error "ytype must be bin or cont"
		error 999
		}

		

* **** 2. Treatment and response rates
	
	
	// Share treatment group E(Z)
	
	qui sum Z if `touse' [`weight'`exp']
	sca EZ=r(mean)
	sca nobs_tot=r(N)

	// Response rates in treatment and control group
	
	foreach z in 0 1 {
		qui sum `R' if `Z'==`z' & `touse' [`weight'`exp']
		sca pr`z'=r(mean)
		sca nobs`z'=r(N)

		qui sum `y' if `Z'==`z' & `R'==1 & `touse' [`weight'`exp']
		sca E`z'=r(mean)
		sca se`z'=r(sd)
		sca nobsR`z'=r(N)
	}
	

* **** 3. Determine whether control or treatment group should be truncated


	if pr1 > pr0 {
		global z = 1
		
		sca ppb = pr0
		sca pqb = pr1
		
		sca Epb = E0
		sca spb = se0
		sca nRpb = nobsR0
		sca nRqb = nobsR1
		
		sca ez=EZ
		}

	else if pr0 > pr1 {
		global z = 0

		sca ppb = pr1
		sca pqb = pr0

		sca Epb = E1
		sca spb = se1
		sca nRpb = nobsR1
		sca nRqb = nobsR0
	
		sca ez=1-EZ		
		}
		
	else {
	dis _n "Response rates are identical in treatment & control groups. Run OLS."
	exit
	}
		


* **** 4. Determine number of calls for truncating 

	//calculate: wmax & wz (wz denotes w1 in Behaghel et al.)
	qui sum `N'
	global wmax = r(max)

	tempvar Rt
	qui gen `Rt'=0 if `Z'==$z & `touse'
	forval w = 1/$wmax {
		qui replace `Rt'=1 if `Z'==$z & `R'==1 & `N'<=`w' & `touse'
		qui sum `Rt' if `Z'==$z & `touse' [`weight'`exp']
		if r(mean) >= ppb {
			global wz = `w'
			continue, break
		}
	}
	

	* Compute ptild and qtild
	* ("ppt" denotes ptild=Pr(N<=w1|Z=1); "pqt" denotes qtild=Pr(N<=w1-1|Z=1))
	tempvar Rpt
	qui gen `Rpt'=0 if `Z'==$z & `touse'
	qui replace `Rpt'=1 if `Z'==$z & `R'==1 & `N'<=$wz
	qui sum `Rpt' if `Z'==$z & `touse' [`weight'`exp']
	sca ppt=r(mean)
	
	tempvar Rqt
	qui gen `Rqt'=0 if `Z'==$z & `touse'
	qui replace `Rqt'=1 if `Z'==$z & `R'==1 & `N'<=$wz-1
	qui sum `Rqt' if `Z'==$z & `touse' [`weight'`exp']
	sca pqt=r(mean)

	* Compute mu
	* "Eqt" denotes mu=E(y|N<=w1-1,Z=1)
	qui sum `y' if `Z'==$z & `Rpt'==1 & `N'<=$wz -1 & `touse' [`weight'`exp']
	sca Eqt=r(mean)
	sca sqt=r(sd)
	sca nRqt=r(N)
	
	* Compute E(Z 1(N=w1))
	tempvar ZNw1
	qui gen `ZNw1'= (`Z'==$z & `N'==$wz) if `touse'
	qui sum `ZNw1' if `touse' [`weight'`exp']
	sca eZNw1=r(mean)

	
	
* **** 5. Lee truncature 	
	
	//calculate: alpha, gamma, y(alpha), y(1-alpha), y_lb, y_ub	
	* aa denotes alpha=(ptild-pbar)/(ptild-qtild)
	sca aa = (ppt-ppb)/(ppt-pqt)
	
	//y binary
	if "`ytype'" == "bin" {
		qui sum `y' if `Z'==$z & `N'==$wz & `touse' [`weight'`exp']
		sca	gm = r(mean)
		sca y_ub = min(1,gm/(1-aa))
		sca y_lb = max(0,(gm-aa)/(1-aa))
		}
	
	//y continuous
	else if "`ytype'" == "cont" {
		tempvar refyc
		qui gen `refyc' = `y' if `Z'==$z & `N'==$wz & `touse'
		
		tempvar var_aa var_yaa
		tempvar var_ma var_yma

		// Compute quantiles of y
		
		qui gen `var_aa' = aa
		invcdf `var_aa' if `Z'==$z & `N'==$wz & `touse' [`weight'`exp'], ref(`refyc') g(`var_yaa') 
		qui sum `var_yaa' if `Z'==$z & `N'==$wz & `touse' [`weight'`exp']
		sca yaa = r(mean)

		sca ma = 1-aa
		qui gen `var_ma' = ma
		invcdf `var_ma' if `Z'==$z & `N'==$wz & `touse' [`weight'`exp'], ref(`refyc') g(`var_yma')
		qui sum `var_yma' if `Z'==$z & `N'==$wz & `touse' [`weight'`exp']
		sca yma = r(mean)
		
		// Compute expectation of y in truncated sample
		
		* "y_lb" denotes E(y|y<=y_(1-a), Z=1, N=w1)
		* see Appendix derivation 
		qui sum `y' if `y'<=yma & `Z'==$z & `N'==$wz & `touse' [`weight'`exp']
		sca y_lb = r(mean)
		sca	Vxlb = r(sd)^2

		* "y_ub" denotes E(y|y>=y_a, Z=1, w=1)
		* see Appendix derivation 
		qui sum `y' if `y'>=yaa & `Z'==$z & `N'==$wz & `touse' [`weight'`exp']
		sca y_ub = r(mean)
		sca Vxub = r(sd)^2
		}
		
	else dis _n "      " "y type invalid"
	

* **** 6. Compute bounds
	
	//calculate: DELTAS
	* see Appendix derivation 
	sca del_lb= pqt/ppb * Eqt + (1 - pqt/ppb) * y_lb - Epb
	sca del_ub = pqt/ppb * Eqt + (1 - pqt/ppb) * y_ub - Epb

	

* **** 7. Compute asymptotic variance for bounds	
	
	//calculate: variance components

	//Vpbar Vptild Vqtild
	
	sca Vppb = ppb*(1-ppb)/(1-ez)
	sca Vpqt = pqt*(1-pqt)/ez
	sca Vppt = ppt*(1-ppt)/ez

	//Va
	sca Va = (ppb-pqt)^2/((ppt-pqt)^4)*Vppt	 ///
				+ (1/(ppt-pqt)^2)*Vppb ///
				+ ((ppt-ppb)*(ppb+ppt-2*pqt)/((ppt-pqt)^4))*Vpqt

	
	//Vyub Vylb
	if "`ytype'" == "bin" {
		if y_lb == 0 sca Vylb = 0
		else sca Vylb = (gm*(1-gm))/(eZNw1*(1-aa)^2) + ((1-gm)/(1-aa)^2)^2 * Va
	
		if y_ub == 1 sca Vyub = 0
		else sca Vyub = (gm*(1-gm))/(eZNw1*(1-aa)^2) + (gm/(1-aa)^2)^2 * Va
		}
	
	else if "`ytype'" == "cont" {
		sca Vylb = Vxlb/(eZNw1*(1-aa)) + (yma-y_lb)^2 * aa/(eZNw1*(1-aa)) + ((yma-y_lb)/(1-aa))^2 * Va
		sca Vyub = Vxub/(eZNw1*(1-aa)) + (yaa-y_ub)^2 * aa/(eZNw1*(1-aa)) + ((yaa-y_ub)/(1-aa))^2 * Va
	}
	
	else dis _n "      " "y type invalid"

	
	//Vc=Var(Y|Z=0,N<=wmax)/(E(1-Z)pbar) and Vy=Var(y|Z=1,N<=w1-1)/(E(Z)qtild)
	sca Vc 	= spb^2/((1-ez)*ppb)
	sca Vy	= sqt^2/(ez*pqt)
	
	
	//calculate: VARIANCES & STANDARD ERRORS
	sca V_dlb = ( (pqt/ppb)^2 * Vy + (1-(pqt/ppb))^2 * Vylb  + Vc + ((1/(ppb^2))*Vpqt + (pqt/(ppb^2))^2*Vppb)*((Eqt - y_lb)^2) ) / nobs_tot 						
	sca V_dub = ( (pqt/ppb)^2 * Vy + (1-(pqt/ppb))^2 * Vyub  + Vc + ((1/(ppb^2))*Vpqt + (pqt/(ppb^2))^2*Vppb)*((Eqt - y_ub)^2) ) / nobs_tot 						

	sca se_dlb = V_dlb^0.5
	sca se_dub = V_dub^0.5

	//calculate: TE confidence interval
	lucbci

	
	//plot: graph	
	if "`graph'" == "graph" plotR `Z' `R' `N' if `touse' [`weight'`exp'], wz($wz)


* **** 8. Display results	
	
	//post results: b & V
	mat rmb = (del_lb, del_ub)
	mat colnames rmb = `Z':lower `Z':upper
    local cn : colfullnames rmb
    //mat list rmb

    mat vrm = (V_dlb, V_dub)
    mat rmV = diag(vrm)
    matrix colnames rmV = `cn'
	matrix rownames rmV = `cn'
    //mat list rmV
    
    ereturn post rmb rmV, depname(`y') esample(`touse') properties(b V) 
	
	//post results: locals & scalars
    ereturn local depvar "`y'"	
	ereturn local title "Behaghel et al (2015) treatment effect bounds"
	ereturn local cmdline `cmdline'
	ereturn local cmd `cmd'

	ereturn scalar N = nRpb + nRqt	
	ereturn scalar z = $z
	ereturn scalar w = $wz
	ereturn scalar dlb = del_lb
	ereturn scalar dub = del_ub
	ereturn scalar level = `c(level)'
	ereturn scalar cilower = cilower
	ereturn scalar ciupper = ciupper
	
	* Additional results that can be of interest for programing
	* ereturn scalar se_dlb=se_dlb
	* ereturn scalar Vc=Vc
	* ereturn scalar Epb=Epb
	* ereturn scalar Vyub=Vyub
	* ereturn scalar Vylb=Vylb
	* ereturn scalar y_ub=y_ub
	* ereturn scalar Va=Va
	* ereturn scalar aa = aa
	* ereturn scalar Vy=Vy
	* ereturn scalar nobs_tot=nobs_tot
	
	* ereturn list
    
	//display: results
	display _n
	display _n as text "Behaghel et al (2015) treatment effect bounds"
    display _n as text "Truncated number of obs." _skip(10) as text  " =  " as result `e(N)'
    display as text "Sample truncated for Z" _skip(12) as text " =  " as result `e(z)'
    display as text "above number of calls w" _skip(11) as text " =  " as result `e(w)'
    display as text "Effect `e(level)'% conf. interval" _skip(09) as text " : [" as result %-06.4f `e(cilower)' _skip(2) as result %-06.4f `e(ciupper)' as text "]"
    display ""

		
    //ereturn post rmb rmV, depname(`y') esample(`touse') properties(b V) 
	ereturn display
		
	//do: housekeeping
    set level `clevel'
	set more `cmore'
end


//calculate TE conf. interval
capture program drop lucbci
program define lucbci
	local cl = invnormal(1-(100-`c(level)')/100)
	local cu = invnormal(1-(100-`c(level)')/200)
	local em = 10^15
	forvalues cc = `cl'(0.001)`cu' {
		//local ee = ((normal(`cc'+(_b[upper]-_b[lower])/max(_se[lower], _se[upper]))-normal(-`cc')) - (1-(100-`c(level)')/100))^2
		local ee = ((normal(`cc'+(del_ub-del_lb)/max(se_dub,se_dlb))-normal(-`cc')) - (1-(100-`c(level)')/100))^2
		if `ee' < `em' {
			local em = `ee'
			local cm = `cc'
		}
	}
	//dis `cl' " " `cu' " " `cm'
	scalar cilower = del_lb - `cm' * se_dlb
	scalar ciupper = del_ub + `cm' * se_dub
end


//plot: calculate response rates at different ncalls
capture program drop plotR
program define plotR
	syntax varlist(min=3 max=3 numeric) [if] [in] [fw aw], wz(int)
	marksample touse, novarlist
	
	set more off
	
	tokenize `varlist'
	local Z `1'
	local R `2'
	local N `3'
	
	qui sum `N'
	local wmax = r(max)

	mata: cR = J(`wmax',3,0)
	tempvar cR0 cR1
	qui gen `cR0'=0
	qui gen `cR1'=0
	forval w = 1/`wmax' {
		mata: cR[`w',1] = `w'
		
		qui replace `cR0'=1 if `R'==1 & `Z'==0 & `N'<=`w' & `touse'
		qui replace `cR1'=1 if `R'==1 & `Z'==1 & `N'<=`w' & `touse'
		
		qui sum `cR0' if `Z'==0 & `touse' [`weight'`exp']
		local mcR = r(mean)
		mata: cR[`w',2] = `mcR'
		
		qui sum `cR1' if `Z'==1 & `touse' [`weight'`exp']
		local mcR = r(mean)
		mata: cR[`w',3] = `mcR'
	}
	
	//dis _n
	//mata cR
	
	mata: w = cR[,1]
	mata: cR0 = cR[,2]
	mata: cR1 = cR[,3]
	
	mata: st_numscalar("R0",cR[`wmax',2])
	mata: st_numscalar("R1",cR[`wmax',3])
	sca ppb = min(R0,R1)
	
	local opts = "yline(" + string(ppb) + ")" ///
				 + "xline(" + string(`wz') + ")" + ///
				 + "xline(" + string(`wz'-1) + ")"

	mata: rplot(cR1, cR0, w, "`opts'")
end

//plot: do the plotting
capture mata: mata drop rplot()	
mata:
void rplot(real colvector cR1, real colvector cR0, real colvector w, | string scalar opts)
{
	real scalar n, N, sCR0, sCR1

	n = rows(cR0)
	if (rows(w)!=n) _error(3200)
	
	stata("preserve")

	N = st_nobs()
	if (N<n) st_addobs(n-N)

	st_store((1,n), sCR0=st_addvar("double", st_tempname()), cR0)
	st_store((1,n), sCR1=st_addvar("double", st_tempname()), cR1)
	st_store((1,n), sw=st_addvar("double", st_tempname()), w)
	
	stata("twoway line " + st_varname(sCR1) + " " + st_varname(sCR0) + " " + st_varname(sw) + "," ///
				+ "xtitle(" + "number of calls" + ")" ///
				+ "legend(lab(1 " + "Z=1" + ") lab(2 " + "Z=0" + "))" ///
				+ opts)

	if (N<n) st_dropobsin((N+1,n))
	
	stata("restore")
}
end

